typedef unsigned char uint8_t;
typedef unsigned int uint32_t;
typedef unsigned long uint64_t;
typedef int int32_t;

#define DEG_TWO 12
#define PARAM_BIG_N (1 << DEG_TWO)
#define NUM_OF_ITER (PARAM_BIG_N / 2)

typedef struct _THE_PAIR
{
	uint32_t Index1;
	uint32_t Index2;
} THE_PAIR;

void Xchg(__global uint32_t Data[PARAM_BIG_N], __global uint32_t Indices[PARAM_BIG_N], uint32_t Ind1, uint32_t Ind2)
{
	uint32_t tmp = Indices[Ind1];
	Indices[Ind1] = Indices[Ind2];
	Indices[Ind2] = tmp;
}

__kernel void Shader(__global uint32_t Data[PARAM_BIG_N], __global uint32_t Indices[PARAM_BIG_N], __global THE_PAIR Result[PARAM_BIG_N], __global uint32_t* Number)
{
	uint32_t GID = (uint32_t)get_global_id(0);
	if(GID < PARAM_BIG_N)
	{
		Indices[GID] = GID;
		if(GID < PARAM_BIG_N / 2)
		{
			uint32_t BaseIndex = 0;
			barrier(CLK_GLOBAL_MEM_FENCE);
			
			for(int32_t i = DEG_TWO-1; i >= 0; i--)
			{
				uint32_t BlockSize = (1 << i);
				uint32_t RelativeIndex = GID - BaseIndex;
				for(uint32_t j = 0; j < BlockSize; j++)
				{
					if(Data[Indices[GID]]>Data[Indices[BaseIndex + BlockSize + ((RelativeIndex + j) % BlockSize)/*GID + BlockSize + j*/]])
					{
						Xchg(Data, Indices, GID, BaseIndex + BlockSize + ((RelativeIndex + j) % BlockSize));
					}
					
					barrier(CLK_GLOBAL_MEM_FENCE);
				}
				uint32_t NewSize = BlockSize / 2;
				if(RelativeIndex >= NewSize)
				{
					GID += NewSize;
					BaseIndex += BlockSize;
				}
			}
		}
	}
}